var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
import React from "react";
import { FormControl, InputLabel, MenuItem, Select, withStyles } from "@material-ui/core";
import { EditorFieldParentType } from "../EditorField";
import EditorField from "../EditorField/EditorField";
import ajv from "ajv";
export var styles = {
    root: {
        width: "100%"
    },
    properties: {
        "margin-left": "17px",
        "list-style": "none",
        margin: "0px",
        padding: "0px"
    },
    property: {},
    rootProperties: {
        "margin-left": "0px"
    },
    rootProperty: {
        "border-left": "4px solid #e5e5e5",
        "padding-left": "15px",
        "margin-bottom": "17px"
    },
    collapsibleBody: {
        padding: "0px"
    },
    header: {
        display: "inline-flex",
        "vertical-align": "middle"
    },
    selectTitle: {
        display: "block",
        fontSize: "14px",
        textTransform: "uppercase",
        padding: 16,
        fontWeight: 500
    }
};
export function isOneOfField(schema) {
    var hasOneOfKeyword = schema && schema.oneOf;
    return hasOneOfKeyword;
}
function detectCommonConstant(oneOf) {
    var constantsPropertiesPerSchema = oneOf.map(function (schema) {
        if (!schema.properties) {
            return [];
        }
        return Object.keys(schema.properties).filter(function (key) { return schema.properties[key].const; });
    });
    var allConstantProperties = constantsPropertiesPerSchema.flatMap(function (x) { return x; });
    var _loop_1 = function (propertyName) {
        var isPropertyFoundOnEachSchema = constantsPropertiesPerSchema.filter(function (x) { return x.indexOf(propertyName) !== -1; })
            .length === oneOf.length;
        if (isPropertyFoundOnEachSchema) {
            return { value: propertyName };
        }
    };
    for (var _i = 0, allConstantProperties_1 = allConstantProperties; _i < allConstantProperties_1.length; _i++) {
        var propertyName = allConstantProperties_1[_i];
        var state_1 = _loop_1(propertyName);
        if (typeof state_1 === "object")
            return state_1.value;
    }
    return null;
}
function getSelectedSchema(oneOf, value) {
    /**
     * If each schema has common const to specifically disambiguate it, prefer to use that
     */
    var commonConstant = detectCommonConstant(oneOf);
    if (commonConstant) {
        var constantValue = value[commonConstant];
        for (var i = 0; i < oneOf.length; i++) {
            var schema = oneOf[i];
            if (schema.properties[commonConstant].const === constantValue) {
                return i;
            }
        }
        return 0;
    }
    /**
     * Try to find a schema that matches the data loosely
     */
    for (var i = 0; i < oneOf.length; i++) {
        var schema = __assign({}, oneOf[i], { required: undefined });
        if (schema.properties) {
            schema.anyOf = Object.keys(schema.properties).map(function (key) {
                return {
                    required: [key]
                };
            });
        }
        if (ajv().validate(schema, value)) {
            return i;
        }
    }
    return 0;
}
var EditorOneOfField = function (props) {
    var schema = props.schema, registry = props.registry, onChange = props.onChange, valueProp = props.value, classes = props.classes, parentType = props.parentType, errorReport = props.errorReport, pointer = props.pointer, required = props.required, disabled = props.disabled, readonly = props.readonly;
    var handleChange = React.useCallback(function (newValue) {
        if (onChange) {
            onChange(newValue);
        }
    }, [onChange, valueProp]);
    var value = valueProp || {};
    var schemaOptions = schema.oneOf || [];
    var schemaOptionTitles = React.useMemo(function () {
        return schemaOptions.map(function (nestedSchema, index) {
            return nestedSchema.title || "Option " + (index + 1);
        });
    }, [schemaOptions]);
    var _a = React.useState(function () {
        return getSelectedSchema(schemaOptions, value);
    }), selectedIndex = _a[0], setSelectedIndex = _a[1];
    var selectedSchema = React.useMemo(function () {
        return schemaOptions[selectedIndex] || {};
    }, [selectedIndex]);
    var handleChangeSelectedIndex = function (event) {
        setSelectedIndex(event.target.value);
    };
    var children = (React.createElement("div", { className: classes.root },
        React.createElement(FormControl, null,
            React.createElement(InputLabel, { required: required }, schema.title || ""),
            React.createElement(Select, { autoWidth: true, disabled: disabled, onChange: handleChangeSelectedIndex, inputProps: {
                    readOnly: readonly,
                    "aria-label": schema.description || ""
                }, value: selectedIndex }, schemaOptionTitles.map(function (title, index) { return (React.createElement(MenuItem, { key: index, value: index }, "" + title)); }))),
        React.createElement(EditorField, { pointer: "" + pointer, errorReport: errorReport, parentType: EditorFieldParentType.ARRAY, schema: selectedSchema, value: value, registry: registry, 
            // tslint:disable-next-line
            onChange: handleChange })));
    return children;
};
export default withStyles(styles, { name: "DcEditorOneOfField" })(EditorOneOfField);
