import React from "react";
import { FormControl, FormHelperText, TextField as MuiTextField, withStyles } from "@material-ui/core";
import { getErrorMessages, getErrorsForPointer } from "../EditorErrorMessages";
export var styles = {
    root: {
        width: "100%"
    }
};
var EditorTextField = function (props) {
    var schema = props.schema, value = props.value, readonly = props.readonly, required = props.required, disabled = props.disabled, onChange = props.onChange, classes = props.classes, pointer = props.pointer, errorReport = props.errorReport, registry = props.registry;
    var handleChange = React.useCallback(function (event) {
        var newValue = event.target.value;
        if (newValue === "") {
            newValue = undefined;
        }
        if (onChange) {
            onChange(newValue);
        }
    }, [onChange]);
    var errors = getErrorsForPointer(pointer, errorReport);
    var errorMessages = getErrorMessages(schema, errors, registry);
    return (React.createElement(FormControl, { className: classes.root },
        React.createElement(MuiTextField, { autoComplete: "off", label: schema.title || "", disabled: disabled, required: required, onChange: handleChange, inputProps: {
                readOnly: readonly,
                "aria-label": schema.description || ""
            }, error: errorMessages.length > 0, value: value || "" }),
        React.createElement(FormHelperText, { error: errorMessages.length > 0 }, errorMessages.length ? errorMessages[0] : schema.description)));
};
export default withStyles(styles, { name: "DcEditorTextField" })(EditorTextField);
